@@ -10,6 +10,7 @@ require 'octokit/repository'
 require 'octokit/user'
 require 'octokit/organization'
 require 'octokit/preview'
+require 'octokit/client/apps'
 require 'octokit/client/authorizations'
 require 'octokit/client/commits'
 require 'octokit/client/commit_comments'
@@ -22,13 +23,13 @@ require 'octokit/client/feeds'
 require 'octokit/client/gists'
 require 'octokit/client/gitignore'
 require 'octokit/client/hooks'
-require 'octokit/client/integrations'
 require 'octokit/client/issues'
 require 'octokit/client/labels'
 require 'octokit/client/legacy_search'
 require 'octokit/client/licenses'
 require 'octokit/client/meta'
 require 'octokit/client/markdown'
+require 'octokit/client/marketplace'
 require 'octokit/client/milestones'
 require 'octokit/client/notifications'
 require 'octokit/client/objects'
@@ -78,13 +79,14 @@ module Octokit
     include Octokit::Client::Gists
     include Octokit::Client::Gitignore
     include Octokit::Client::Hooks
-    include Octokit::Client::Integrations
+    include Octokit::Client::Apps
     include Octokit::Client::Issues
     include Octokit::Client::Labels
     include Octokit::Client::LegacySearch
     include Octokit::Client::Licenses
     include Octokit::Client::Meta
     include Octokit::Client::Markdown
+    include Octokit::Client::Marketplace
     include Octokit::Client::Milestones
     include Octokit::Client::Notifications
     include Octokit::Client::Objects
