@@ -31,6 +31,8 @@ module Octokit
       # @option values [String] :email Publicly visible email address.
       # @option values [String] :location Location of organization.
       # @option values [String] :name GitHub username for organization.
+      # @option values [String] :default_repository_permission The default permission members have on organization repositories.
+      # @option values [Boolean] :members_can_create_repositories Set true to allow members to create repositories on the organization.
       # @return [Sawyer::Resource] Hash representing GitHub organization.
       # @see https://developer.github.com/v3/orgs/#edit-an-organization
       # @example
@@ -44,7 +46,7 @@ module Octokit
       # @example
       #   @client.update_org('github', {:company => 'Unicorns, Inc.'})
       def update_organization(org, values, options = {})
-        patch Organization.path(org), options.merge({:organization => values})
+        patch Organization.path(org), options.merge(values)
       end
       alias :update_org :update_organization
 
@@ -62,6 +64,7 @@ module Octokit
       # @param user [Integer, String] GitHub user login or id of the user to get
       #   list of organizations.
       # @return [Array<Sawyer::Resource>] Array of hashes representing organizations.
+      # @see https://developer.github.com/v3/orgs/#list-your-organizations
       # @see https://developer.github.com/v3/orgs/#list-user-organizations
       # @example
       #   Octokit.organizations('pengwynn')
@@ -76,12 +79,29 @@ module Octokit
       # @example
       #   @client.organizations
       def organizations(user=nil, options = {})
-        get "#{User.path user}/orgs", options
+        paginate "#{User.path user}/orgs", options
       end
       alias :list_organizations :organizations
       alias :list_orgs :organizations
       alias :orgs :organizations
 
+      # List all GitHub organizations
+      #
+      # This provides a list of every organization, in the order that they
+      # were created.
+      #
+      # @param options [Hash] Optional options.
+      # @option options [Integer] :since The integer ID of the last
+      # Organization that you’ve seen.
+      #
+      # @see https://developer.github.com/v3/orgs/#list-all-organizations
+      #
+      # @return [Array<Sawyer::Resource>] List of GitHub organizations.
+      def all_organizations(options = {})
+        paginate "organizations", options
+      end
+      alias :all_orgs :all_organizations
+
       # List organization repositories
       #
       # Public repositories are available without authentication. Private repos
@@ -189,6 +209,65 @@ module Octokit
       end
       alias :org_public_member? :organization_public_member?
 
+      # List pending organization invitations
+      #
+      # Requires authenticated organization member.
+      #
+      # @param org [String, Integer] Organization GitHub login or id.
+      # @return [Array<Sawyer::Resource>] Array of hashes representing invitations.
+      # @see https://developer.github.com/v3/orgs/members/#list-pending-organization-invitations
+      #
+      # @example
+      #   @client.organization_invitations('github')
+      def organization_invitations(org, options = {})
+        get "#{Organization.path org}/invitations", options
+      end
+      alias :org_invitations :organization_invitations
+
+      # List outside collaborators for an organization
+      #
+      # Requires authenticated organization members.
+      #
+      # @param org [String, Integer] Organization GitHub login or id.
+      # @return [Array<Sawyer::Resource>] Array of hashes representing users.
+      # @see https://developer.github.com/v3/orgs/outside-collaborators/#list-outside-collaborators
+      #
+      # @example
+      #   @client.outside_collaborators('github')
+      def outside_collaborators(org, options={})
+        get "#{Organization.path org}/outside_collaborators", options
+      end
+
+      # Remove outside collaborator from an organization
+      #
+      # Requires authenticated organization members.
+      #
+      # @param org [String, Integer] Organization GitHub login or id.
+      # @param user [String] GitHub username to be removed as outside collaborator
+      # @return [Boolean] Return true if outside collaborator removed from organization, false otherwise.
+      # @see https://developer.github.com/v3/orgs/outside-collaborators/#remove-outside-collaborator
+      #
+      # @example
+      #   @client.remove_outside_collaborator('github', 'lizzhale')
+      def remove_outside_collaborator(org, user, options={})
+        boolean_from_response :delete, "#{Organization.path org}/outside_collaborators/#{user}", options
+      end
+
+      # Converts an organization member to an outside collaborator
+      #
+      # Requires authenticated organization members.
+      #
+      # @param org [String, Integer] Organization GitHub login or id.
+      # @param user [String] GitHub username to be removed as outside collaborator
+      # @return [Boolean] Return true if outside collaborator removed from organization, false otherwise.
+      # @see https://developer.github.com/v3/orgs/outside-collaborators/#convert-member-to-outside-collaborator
+      #
+      # @example
+      #   @client.convert_to_outside_collaborator('github', 'lizzhale')
+      def convert_to_outside_collaborator(org, user, options={})
+        boolean_from_response :put, "#{Organization.path org}/outside_collaborators/#{user}", options
+      end
+
       # List teams
       #
       # Requires authenticated organization member.
@@ -212,21 +291,22 @@ module Octokit
       # @param org [String, Integer] Organization GitHub login or id.
       # @option options [String] :name Team name.
       # @option options [Array<String>] :repo_names Repositories for the team.
-      # @option options [String, optional] :permission ('pull') Permissions the
-      #   team has for team repositories.
-      #
-      #   `pull` - team members can pull, but not push to or administer these repositories.
-      #   `push` - team members can pull and push, but not administer these repositories.
-      #   `admin` - team members can pull, push and administer these repositories.
+      # @option options [Array<String>] :maintainers Maintainers for the team.
+      # @option options [Integer] :parent_team_id ID of a team to set as the parent team.
       # @return [Sawyer::Resource] Hash representing new team.
       # @see https://developer.github.com/v3/orgs/teams/#create-team
       # @example
       #   @client.create_team('github', {
       #     :name => 'Designers',
-      #     :repo_names => ['github/dotfiles'],
-      #     :permission => 'push'
+      #     :repo_names => ['github/dotfiles']
       #   })
       def create_team(org, options = {})
+        if options.key?(:permission)
+          octokit_warn "Deprecated: Passing :permission option to #create_team. Assign team repository permission by passing :permission to #add_team_repository instead."
+        end
+        if options.key?(:parent_team_id)
+          options = ensure_api_media_type(:nested_teams, options)
+        end
         post "#{Organization.path org}/teams", options
       end
 
@@ -243,6 +323,20 @@ module Octokit
         get "teams/#{team_id}", options
       end
 
+      # List child teams
+      #
+      # Requires authenticated organization member.
+      #
+      # @param team_id [Integer] Team id.
+      # @return [Sawyer::Resource] Hash representing team.
+      # @see https://developer.github.com/v3/orgs/teams/#list-child-teams
+      # @example
+      #   @client.child_teams(100000, :accept => "application/vnd.github.hellcat-preview+json")
+      def child_teams(team_id, options = {})
+        options = ensure_api_media_type(:nested_teams, options)
+        paginate "teams/#{team_id}/teams", options
+      end
+
       # Update team
       #
       # Requires authenticated organization owner.
@@ -254,6 +348,7 @@ module Octokit
       #   `pull` - team members can pull, but not push to or administer these repositories.
       #   `push` - team members can pull and push, but not administer these repositories.
       #   `admin` - team members can pull, push and administer these repositories.
+      # @option options [Integer] :parent_team_id ID of a team to set as the parent team.
       # @return [Sawyer::Resource] Hash representing updated team.
       # @see https://developer.github.com/v3/orgs/teams/#edit-team
       # @example
@@ -262,6 +357,9 @@ module Octokit
       #     :permission => 'push'
       #   })
       def update_team(team_id, options = {})
+        if options.key?(:parent_team_id)
+          options = ensure_api_media_type(:nested_teams, options)
+        end
         patch "teams/#{team_id}", options
       end
 
@@ -347,12 +445,26 @@ module Octokit
       # @see https://developer.github.com/v3/orgs/teams/#get-team-member
       #
       # @example Check if a user is in your team
-      #   @client.team_member?('your_team', 'pengwynn')
+      #   @client.team_member?(100000, 'pengwynn')
       #   => false
       def team_member?(team_id, user, options = {})
         boolean_from_response :get, "teams/#{team_id}/members/#{user}", options
       end
 
+      # List pending team invitations
+      #
+      # Requires authenticated organization member.
+      #
+      # @param team_id [Integer] Team id.
+      # @return [Array<Sawyer::Resource>] Array of hashes representing invitations.
+      # @see https://developer.github.com/v3/orgs/teams/#list-pending-team-invitations
+      #
+      # @example
+      #   @client.team_invitations('github')
+      def team_invitations(team_id, options = {})
+        get "teams/#{team_id}/invitations", options
+      end
+
       # List team repositories
       #
       # Requires authenticated organization member.
@@ -376,7 +488,7 @@ module Octokit
       # @return [Boolean] True if managed by a team. False if not managed by
       #   the team OR the requesting user does not have authorization to access
       #   the team information.
-      # @see https://developer.github.com/v3/orgs/teams/#get-team-repo
+      # @see https://developer.github.com/v3/orgs/teams/#check-if-a-team-manages-a-repository
       # @example
       #   @client.team_repository?(8675309, 'octokit/octokit.rb')
       # @example
@@ -388,19 +500,28 @@ module Octokit
 
       # Add team repository
       #
+      # This can also be used to update the permission of an existing team
+      #
       # Requires authenticated user to be an owner of the organization that the
       # team is associated with. Also, the repo must be owned by the
       # organization, or a direct form of a repo owned by the organization.
       #
       # @param team_id [Integer] Team id.
       # @param repo [String, Hash, Repository] A GitHub repository.
+      # @option options [String] :permission The permission to grant the team.
+      #   Only valid on organization-owned repositories.
+      #   Can be one of: <tt>pull</tt>, <tt>push</tt>, or <tt>admin</tt>.
+      #   If not specified, the team's <tt>permission</tt> attribute will be
+      #   used to determine what permission to grant the team on this repository.
       # @return [Boolean] True if successful, false otherwise.
       # @see Octokit::Repository
-      # @see https://developer.github.com/v3/orgs/teams/#add-team-repo
+      # @see https://developer.github.com/v3/orgs/teams/#add-or-update-team-repository
       # @example
       #   @client.add_team_repository(100000, 'github/developer.github.com')
       # @example
       #   @client.add_team_repo(100000, 'github/developer.github.com')
+      # @example Add a team with admin permissions
+      #   @client.add_team_repository(100000, 'github/developer.github.com', permission: 'admin')
       def add_team_repository(team_id, repo, options = {})
         boolean_from_response :put, "teams/#{team_id}/repos/#{Repository.new(repo)}", options.merge(:name => Repository.new(repo))
       end
@@ -416,7 +537,7 @@ module Octokit
       # @param repo [String, Hash, Repository] A GitHub repository.
       # @return [Boolean] Return true if repo removed from team, false otherwise.
       # @see Octokit::Repository
-      # @see https://developer.github.com/v3/orgs/teams/#remove-team-repo
+      # @see https://developer.github.com/v3/orgs/teams/#remove-team-repository
       # @example
       #   @client.remove_team_repository(100000, 'github/developer.github.com')
       # @example
@@ -481,7 +602,7 @@ module Octokit
       # @return [Array<Sawyer::Resource>] Array of team resources.
       # @see https://developer.github.com/v3/orgs/teams/#list-user-teams
       def user_teams(options = {})
-        paginate "/user/teams", options
+        paginate "user/teams", options
       end
 
       # Check if a user has a team membership.
@@ -506,7 +627,7 @@ module Octokit
       #
       # @return [Sawyer::Resource] Hash of team membership info
       #
-      # @see https://developer.github.com/v3/orgs/teams/#add-team-membership
+      # @see https://developer.github.com/v3/orgs/teams/#add-or-update-team-membership
       #
       # @example Check if a user has a membership for a team
       #   @client.add_team_membership(1234, 'pengwynn')
@@ -535,26 +656,130 @@ module Octokit
       end
       alias :org_memberships :organization_memberships
 
-      # Get an organization membership for the authenticated user
+      # Get an organization membership
       #
-      # @param org [String] Organization GitHub login.
+      # @param org [Integer, String] The GitHub Organization.
+      # @option options [String] :user  The login of the user, otherwise authenticated user.
       # @return [Sawyer::Resource] Hash representing the organization membership.
       # @see https://developer.github.com/v3/orgs/members/#get-your-organization-membership
+      # @see https://developer.github.com/v3/orgs/members/#get-organization-membership
       def organization_membership(org, options = {})
-        get "user/memberships/orgs/#{org}", options
+        if user = options.delete(:user)
+          get "#{Organization.path(org)}/memberships/#{user}", options
+        else
+          get "user/memberships/orgs/#{org}", options
+        end
       end
       alias :org_membership :organization_membership
 
-      # Edit an organization membership for the authenticated user
+      # Edit an organization membership
       #
       # @param org [String] Organization GitHub login.
+      # @option options [String] :role  The role of the user in the organization.
       # @option options [String] :state The state that the membership should be in.
+      # @option options [String] :user  The login of the user, otherwise authenticated user.
       # @return [Sawyer::Resource] Hash representing the updated organization membership.
       # @see https://developer.github.com/v3/orgs/members/#edit-your-organization-membership
+      # @see https://developer.github.com/v3/orgs/members/#add-or-update-organization-membership
       def update_organization_membership(org, options = {})
-        patch "user/memberships/orgs/#{org}", options
+        if user = options.delete(:user)
+          put "orgs/#{org}/memberships/#{user}", options
+        else
+          patch "user/memberships/orgs/#{org}", options
+        end
       end
       alias :update_org_membership :update_organization_membership
+
+      # Remove an organization membership
+      #
+      # @param org [String] Organization GitHub login.
+      # @return [Boolean] Success
+      # @see https://developer.github.com/v3/orgs/members/#remove-organization-membership
+      def remove_organization_membership(org, options = {})
+        user = options.delete(:user)
+        user && boolean_from_response(:delete, "orgs/#{org}/memberships/#{user}", options)
+      end
+
+      # Initiates the generation of a migration archive.
+      #
+      # Requires authenticated organization owner.
+      #
+      # @param org [String, Integer] Organization GitHub login or id.
+      # @param repositories [Array<String>] :repositories Repositories for the organization.
+      # @option options [Boolean, optional] :lock_repositories Indicates whether repositories should be locked during migration
+      # @return [Sawyer::Resource] Hash representing the new migration.
+      # @example
+      #   @client.start_migration('github', ['github/dotfiles'])
+      # @see https://developer.github.com/v3/orgs/migrations/#start-a-migration
+      def start_migration(org, repositories, options = {})
+        options = ensure_api_media_type(:migrations, options)
+        options[:repositories] = repositories
+        post "orgs/#{org}/migrations", options
+      end
+
+      # Lists the most recent migrations.
+      #
+      # Requires authenticated organization owner.
+      #
+      # @param org [String, Integer] Organization GitHub login or id.
+      # @return [Array<Sawyer::Resource>] Array of migration resources.
+      # @see https://developer.github.com/v3/orgs/migrations/#get-a-list-of-migrations
+      def migrations(org, options = {})
+        options = ensure_api_media_type(:migrations, options)
+        paginate "orgs/#{org}/migrations", options
+      end
+
+      # Fetches the status of a migration.
+      #
+      # Requires authenticated organization owner.
+      #
+      # @param org [String, Integer] Organization GitHub login or id.
+      # @param id [Integer] ID number of the migration.
+      # @see https://developer.github.com/v3/orgs/migrations/#get-the-status-of-a-migration
+      def migration_status(org, id, options = {})
+        options = ensure_api_media_type(:migrations, options)
+        get "orgs/#{org}/migrations/#{id}", options
+      end
+
+      # Fetches the URL to a migration archive.
+      #
+      # Requires authenticated organization owner.
+      #
+      # @param org [String, Integer] Organization GitHub login or id.
+      # @param id [Integer] ID number of the migration.
+      # @see https://developer.github.com/v3/orgs/migrations/#download-a-migration-archive
+      def migration_archive_url(org, id, options = {})
+        options = ensure_api_media_type(:migrations, options)
+        url = "orgs/#{org}/migrations/#{id}/archive"
+
+        response = client_without_redirects(options).get(url)
+        response.headers['location']
+      end
+
+      # Deletes a previous migration archive.
+      #
+      # Requires authenticated organization owner.
+      #
+      # @param org [String, Integer] Organization GitHub login or id.
+      # @param id [Integer] ID number of the migration.
+      # @see https://developer.github.com/v3/orgs/migrations/#delete-a-migration-archive
+      def delete_migration_archive(org, id, options = {})
+        options = ensure_api_media_type(:migrations, options)
+        delete "orgs/#{org}/migrations/#{id}/archive", options
+      end
+
+      # Unlock a previous migration archive.
+      #
+      # Requires authenticated organization owner.
+      #
+      # @param org [String, Integer] Organization GitHub login or id.
+      # @param id [Integer] ID number of the migration.
+      # @param repo [String] Name of the repository.
+      # @see https://developer.github.com/v3/orgs/migrations/#unlock-a-repository
+      def unlock_repository(org, id, repo, options = {})
+        options = ensure_api_media_type(:migrations, options)
+        delete "orgs/#{org}/migrations/#{id}/repos/#{repo}/lock", options
+      end
     end
   end
 end
